use axum::{
    extract::{Multipart, Path, State},
    http::StatusCode,
    response::{Html, IntoResponse, Redirect},
};
use sqlx::SqlitePool;
use std::fmt::Write;
use tokio::io::AsyncWriteExt;
use tracing::{error, info, instrument, warn};

#[allow(clippy::unwrap_used)]
#[instrument(skip_all)]
pub async fn upload_script(
    // State(database): State<SqlitePool>,
    mut multipart: Multipart,
) -> impl IntoResponse {
    while let Some(field) = multipart.next_field().await.unwrap() {
        // Borrow file_name as &str
        let file_name = field.file_name().map(|name| name.to_string());

        // Now you can move or consume `field` safely
        let data = field.bytes().await.unwrap();

        if let Some(name) = file_name {
            let _ = tokio::fs::create_dir("./uploads").await;
            // Use `name` (owned String) and `data` here
            let path = format!("./uploads/{}", name);
            let mut file = tokio::fs::File::create(path).await.unwrap();
            file.write_all(&data).await.unwrap();
            // tokio::fs::write(path, &data).await.unwrap();
        }
    }
    Html("File uploaded successfully")
}

#[instrument(skip_all, fields(id = ?Path(&id)))]
pub async fn delete_script(
    State(database): State<SqlitePool>,
    Path(id): Path<String>,
) -> impl IntoResponse {
    info!("deleting");
    if let Err(e) = crate::db::delete_script(&database, id).await {
        warn!(err = e.to_string(), "delete_script");
        return Err(StatusCode::NOT_FOUND);
    }

    Ok(Redirect::to("/"))
}

#[instrument(skip_all, fields(id = ?Path(&id)))]
pub async fn get_script_by_id(
    State(database): State<SqlitePool>,
    Path(id): Path<String>,
) -> impl IntoResponse {
    info!("retrieving");
    let script = match crate::db::get_script(&database, id).await {
        Ok(script) => script,
        Err(e) => {
            warn!(err = e.to_string(), "get_script");
            return Err(StatusCode::NOT_FOUND);
        }
    };

    let output = format!(
        r##"<section id="id-{0}"><h1>{1}</h1><div id="id-{0}-parameters">{2}</div><button hx-push-url="/" hx-target="body" hx-swap="outerHTML" hx-delete="/scripts/{0}">DELETE</button></section>"##,
        script.0,
        script.1.name(),
        script
            .1
            .parameters()
            .iter()
            .fold(String::new(), |mut acc, x| {
                if let Err(e) = write!(acc, "<p>{:?}</p>", x) {
                    error!(err = e.to_string(), script = ?x, "failed to write parameter");
                    return String::new();
                }
                acc
            }),
    );

    Ok(Html(output))
}

#[instrument(skip_all)]
pub async fn get_scripts(State(database): State<SqlitePool>) -> impl IntoResponse {
    info!("listing");
    let scripts = match crate::db::list_scripts(&database).await {
        Ok(scripts) => scripts,
        Err(e) => {
            error!(err = e, "list_scripts");
            return Err(StatusCode::INTERNAL_SERVER_ERROR);
        }
    };

    let content = scripts.iter().fold(String::new(), |mut acc, x| {
        if let Err(e) = write!(
            acc,
            r##"<li hx-get="/scripts/{0}" hx-push-url="/scripts/{0}" hx-target="#scripts" hx-swap="outerHTML">{1} [{0}]</li>"##,
            x.0,
            x.1.name()
        ) {
            error!(err = e.to_string(), script=?x ,"failed to write script");
            return String::new();
        }
        // if let Err(e) = write!(acc, "{} [", x.1.name()) {
        //     error!(err = e.to_string(), script=?x ,"failed to write id");
        //     return String::new();
        // }
        // if let Err(e) = write!(acc, "{}]</li>", x.0) {
        //     error!(err = e.to_string(), script=?x ,"failed to write id");
        //     return String::new();
        // }
        acc
    });

    let output = format!(
        r##"<section id="scripts"><ul>{content}</ul><button hx-get="/scripts" hx-swap="outerHTML" hx-target="#scripts">Refresh</button><form hx-post="/scripts" hx-target="#upload-status" enctype="multipart/form-data" method="post">
  <input type="file" name="file" />
  <button type="submit">Upload</button>
</form>
<div id="upload-status"></div>
</section>"##
    );

    Ok(Html(output))
}
